@@ -8,6 +8,7 @@
 #' @param stdin Standard input, NULL to ignore.
 #' @param stdout Standard output, NULL to ignore, TRUE for temp file.
 #' @param stderr Standard error, NULL to ignore, TRUE for temp file.
+#' @param pty Whether we create a PTY.
 #' @param connections Connections to inherit in the child process.
 #' @param poll_connection Whether to create a connection for polling.
 #' @param env Environment vaiables.
@@ -22,9 +23,9 @@
 #' @keywords internal
 
 process_initialize <- function(self, private, command, args,
-                               stdin, stdout, stderr, connections,
-                               poll_connection, env, cleanup, cleanup_tree,
-                               wd, echo_cmd, supervise,
+                               stdin, stdout, stderr, pty, pty_options,
+                               connections, poll_connection, env, cleanup,
+                               cleanup_tree, wd, echo_cmd, supervise,
                                windows_verbatim_args, windows_hide_window,
                                encoding, post_process) {
 
@@ -36,6 +37,8 @@ process_initialize <- function(self, private, command, args,
     is_string_or_null(stdin),
     is_string_or_null(stdout),
     is_string_or_null(stderr),
+    is_flag(pty),
+    is.list(pty_options), is_named(pty_options),
     is_connection_list(connections),
     is.null(poll_connection) || is_flag(poll_connection),
     is.null(env) || is_named_character(env),
@@ -54,6 +57,33 @@ process_initialize <- function(self, private, command, args,
     cleanup <- TRUE
   }
 
+  if (pty && os_type() != "unix") {
+    throw(new_error("`pty = TRUE` is only implemented on Unix"))
+  }
+  if (pty && tolower(Sys.info()[["sysname"]]) == "sunos") {
+    throw(new_error("`pty = TRUE` is not (yet) implemented on Solaris"))
+  }
+  if (pty && !is.null(stdin)) {
+    throw(new_error("`stdin` must be `NULL` if `pty == TRUE`"))
+  }
+  if (pty && !is.null(stdout)) {
+    throw(new_error("`stdout` must be `NULL` if `pty == TRUE`"))
+  }
+  if (pty && !is.null(stderr)) {
+    throw(new_error("`stderr` must be `NULL` if `pty == TRUE`"))
+  }
+
+  def <- default_pty_options()
+  pty_options <- utils::modifyList(def, pty_options)
+  if (length(bad <- setdiff(names(def), names(pty_options)))) {
+    throw(new_error("Uknown pty option(s): ",
+                    paste(paste0("`", bad, "`"), collapse = ", ")))
+  }
+  pty_options <- pty_options[names(def)]
+
+  command <- enc2utf8(command)
+  args <- enc2utf8(args)
+
   private$command <- command
   private$args <- args
   private$cleanup <- cleanup
@@ -62,6 +92,8 @@ process_initialize <- function(self, private, command, args,
   private$pstdin <- stdin
   private$pstdout <- stdout
   private$pstderr <- stderr
+  private$pty <- pty
+  private$pty_options <- pty_options
   private$connections <- connections
   private$env <- env
   private$echo_cmd <- echo_cmd
@@ -89,10 +121,10 @@ process_initialize <- function(self, private, command, args,
   if (!is.null(wd)) {
     wd <- normalizePath(wd, winslash = "\\", mustWork = FALSE)
   }
-  private$status <- .Call(
+  private$status <- rethrow_call(
     c_processx_exec,
-    command, c(command, args), stdin, stdout, stderr, connections, env,
-    windows_verbatim_args, windows_hide_window,
+    command, c(command, args), stdin, stdout, stderr, pty, pty_options,
+    connections, env, windows_verbatim_args, windows_hide_window,
     private, cleanup, wd, encoding,
     paste0("PROCESSX_", private$tree_id, "=YES")
   )
@@ -103,7 +135,8 @@ process_initialize <- function(self, private, command, args,
   ## macOS and Windows and on other OSes it returns 0.0, so we just use the
   ## current time instead. (In the C process handle, there will be 0,
   ## still.)
-  private$starttime <- .Call(c_processx__proc_start_time, private$status)
+  private$starttime <-
+    rethrow_call(c_processx__proc_start_time, private$status)
   if (private$starttime == 0) private$starttime <- Sys.time()
 
   ## Need to close this, otherwise the child's end of the pipe
