@@ -22,63 +22,67 @@ process_has_poll_connection <- function(self, private) {
 process_get_input_connection <- function(self, private) {
   "!DEBUG process_get_input_connection `private$get_short_name()`"
   if (!self$has_input_connection())
-    stop("stdin is not a pipe.")
+    throw(new_error("stdin is not a pipe."))
   private$stdin_pipe
 }
 
 process_get_output_connection <- function(self, private) {
   "!DEBUG process_get_output_connection `private$get_short_name()`"
   if (!self$has_output_connection())
-    stop("stdout is not a pipe.")
+    throw(new_error("stdout is not a pipe."))
   private$stdout_pipe
 }
 
 process_get_error_connection <- function(self, private) {
   "!DEBUG process_get_error_connection `private$get_short_name()`"
   if (!self$has_error_connection())
-    stop("stderr is not a pipe.")
+    throw(new_error("stderr is not a pipe."))
   private$stderr_pipe
 }
 
 process_get_poll_connection <- function(self, private) {
   "!DEBUG process_get_poll_connection `private$get_short_name()`"
-  if (!self$has_poll_connection()) stop("No poll connection")
+  if (!self$has_poll_connection()) throw(new_error("No poll connection"))
   private$poll_pipe
 }
 
 process_read_output <- function(self, private, n) {
   "!DEBUG process_read_output `private$get_short_name()`"
   con <- process_get_output_connection(self, private)
-  .Call(c_processx_connection_read_chars, con, n)
+  if (private$pty) if (poll(list(con), 0)[[1]] == "timeout") return("")
+  rethrow_call(c_processx_connection_read_chars, con, n)
 }
 
 process_read_error <- function(self, private, n) {
   "!DEBUG process_read_error `private$get_short_name()`"
   con <- process_get_error_connection(self, private)
-  .Call(c_processx_connection_read_chars, con, n)
+  rethrow_call(c_processx_connection_read_chars, con, n)
 
 }
 
 process_read_output_lines <- function(self, private, n) {
   "!DEBUG process_read_output_lines `private$get_short_name()`"
   con <- process_get_output_connection(self, private)
-  .Call(c_processx_connection_read_lines, con, n)
+  if (private$pty) {
+    throw(new_error("Cannot read lines from a pty (see manual)"))
+  }
+  rethrow_call(c_processx_connection_read_lines, con, n)
 }
 
 process_read_error_lines <- function(self, private, n) {
   "!DEBUG process_read_error_lines `private$get_short_name()`"
   con <- process_get_error_connection(self, private)
-  .Call(c_processx_connection_read_lines, con, n)
+  rethrow_call(c_processx_connection_read_lines, con, n)
 }
 
 process_is_incompelete_output <- function(self, private) {
   con <- process_get_output_connection(self, private)
-  ! .Call(c_processx_connection_is_eof, con)
+  ! rethrow_call(c_processx_connection_is_eof, con)
 }
 
 process_is_incompelete_error <- function(self, private) {
   con <- process_get_error_connection(self, private)
-  ! .Call(c_processx_connection_is_eof, con)
+  ! rethrow_call(c_processx_connection_is_eof, con)
 }
 
 process_read_all_output <- function(self, private) {
@@ -124,7 +128,7 @@ process_write_input <- function(self, private, str, sep) {
     pstr <- paste(str, collapse = sep)
     str <- iconv(pstr, "", private$encoding, toRaw = TRUE)[[1]]
   }
-  invisible(.Call(c_processx_connection_write_bytes, con, str))
+  invisible(rethrow_call(c_processx_connection_write_bytes, con, str))
 }
 
 process_get_input_file <- function(self, private) {
